#+ELEVENTY_COLLECTIONS: _posts
#+ELEVENTY_BASE_DIR: ~/sync/static-blog/
#+ELEVENTY_BASE_URL: http://sachachua.com
#+TITLE: Post drafts
* Tweaking my 11ty blog to link to the Mastodon post defined in an Org Mode property :11ty:mastodon:org:
:PROPERTIES:
:EXPORT_DATE: 2025-03-27T13:20:15-0400
:EXPORT_ELEVENTY_PERMALINK: /blog/2025/03/tweaking-my-11ty-blog-to-link-to-the-mastodon-post-defined-in-an-org-mode-property/
:EXPORT_ELEVENTY_FILE_NAME:  blog/2025/03/tweaking-my-11ty-blog-to-link-to-the-mastodon-post-defined-in-an-org-mode-property/
:EXPORT_MASTODON: http://social.sachachua.com/@sacha/statuses/01JQCB0QP0S3XPJ81EP4CK8XD9
:END:

One of the things I like about blogging from Org
Mode in Emacs is that it's easy to add properties
to the section that I'm working on and then use
those property values elsewhere. For example, I've
modified Emacs to simplify [[http://sachachua.com/blog/2025/03/mastodon-el-copy-toot-url-after-posting-also-copying-just-this-post-with-11ty/#:~:text=If%20it%20has%20and%20I%20don't%20have%20a%20Mastodon%20toot%20field%20yet%2C%20maybe%20I%20can%20automatically%20set%20that%20property%2C%20assuming%20I%20end%20up%20back%20in%20the%20Org%20Mode%20file%20I%20started%20it%20from.][tooting a link to my
blog post]] and [[http://sachachua.com/blog/2025/03/mastodon-el-copy-toot-url-after-posting-also-copying-just-this-post-with-11ty/#:~:text=If%20it%20has%20and%20I%20don't%20have%20a%20Mastodon%20toot%20field%20yet%2C%20maybe%20I%20can%20automatically%20set%20that%20property%2C%20assuming%20I%20end%20up%20back%20in%20the%20Org%20Mode%20file%20I%20started%20it%20from.][saving the Mastodon status URL]] in
the ~EXPORT_MASTODON~ property. Then I can use
that in my [[http://www.11ty.dev/][11ty]] static site generation process to
include a link to the Mastodon thread as a comment
option.

First, I need to export the property and include
it in the front matter. I use [[http://www.11ty.dev/docs/data-template-dir/][.11tydata.json]] files
to store the details for each blog post. I
modified [[ox-11ty:ox-11ty.el][ox-11ty.el]] so that I could specify
functions to change the front matter
(~org-11ty-front-matter-functions~,
~org-11ty--front-matter~):

#+begin_src elisp :eval no
(defvar org-11ty-front-matter-functions nil
	"Functions to call with the current front matter plist and info.")
(defun org-11ty--front-matter (info)
  "Return front matter for INFO."
  (let* ((date (plist-get info :date))
         (title (plist-get info :title))
         (modified (plist-get info :modified))
         (permalink (plist-get info :permalink))
         (categories (plist-get info :categories))
         (collections (plist-get info :collections))
				 (extra (if (plist-get info :extra) (json-parse-string
																						 (plist-get info :extra)
																						 :object-type 'plist))))
		(seq-reduce
		 (lambda (prev val)
			 (funcall val prev info))
		 org-11ty-front-matter-functions
		 (append
			extra
			(list :permalink permalink
						:date (if (listp date) (car date) date)
						:modified (if (listp modified) (car modified) modified)
						:title (if (listp title) (car title) title)
						:categories (if (stringp categories) (split-string categories) categories)
						:tags (if (stringp collections) (split-string collections) collections))))))
#+end_src

Then I added [[dotemacs:org-mode-publishing-11ty-static-site-generation-include-mastodon-field-in-front-matter][the ~EXPORT_MASTODON~ Org property]] as
part of the front matter. This took a little
figuring out because I needed to pass it as one of
~org-export-backend-options~, where the parameter
is defined as ~MASTODON~ but the actual property
needs to be called ~EXPORT_MASTODON~.

#+begin_src emacs-lisp
(defun my-org-11ty-add-mastodon-to-front-matter (front-matter info)
	(plist-put front-matter :mastodon (plist-get info :mastodon)))
(with-eval-after-load 'ox-11ty
	(cl-pushnew
	 '(:mastodon "MASTODON" nil nil)
	 (org-export-backend-options (org-export-get-backend '11ty)))
	(add-hook 'org-11ty-front-matter-functions #'my-org-11ty-add-mastodon-to-front-matter))
#+end_src

Then I added the Mastodon field as an option to my
[[11ty:_includes/shortcodes/comments.cjs][comments.cjs]] shortcode. This was a little tricky
because I'm not sure I'm passing the data
correctly to the shortcode (sometimes it ends up
as item.data, sometimes it's item.data.data,
...?), but with ~?.~, I can just throw all the
possibilities in there and it'll eventually find
the right one.

#+INCLUDE: ~/sync/static-blog/_includes/shortcodes/comments.cjs src js

I included it in my [[11ty:_includes/shortcodes/post.cjs][post.cjs]] shortcode:

#+INCLUDE: ~/sync/static-blog/_includes/shortcodes/post.cjs src js

I also included it in my [[11ty:_includes/shortcodes/rssItem.cjs][RSS item]] template to make
it easier for people to send me comments without
having to dig through my website for contact info.

#+INCLUDE: ~/sync/static-blog/_includes/shortcodes/rssItem.cjs src js

The new workflow I'm trying out seems to be working:

1. Keep ~npx eleventy --serve~ running in the
   background, using ~.eleventyignore~ to make
   rebuilds reasonably fast.
2. Export the subtree with ~C-c e s 1 1~, which uses ~org-export-dispatch~ to call ~my-org-11ty-export~ with the subtree.
3. After about 10 seconds, use ~my-org-11ty-copy-just-this-post~ and verify.
4. Use ~my-mastodon-11ty-toot-post~ to compose a toot. Edit the toot and post it.
5. Check that the ~EXPORT_MASTODON~ property has been set.
6. Export the subtree again, this time with the front matter.
7. Publish my whole blog.

Next, I'm thinking of modifying
~my-mastodon-11ty-toot-post~ so that it includes a
list of links to blog posts I might be building on
or responding to, and possibly the handles of
people related to those blog posts or topics.
Hmm...

